<!-- VBSortedList.xslt -->
<!-- This code leveraged from Joel Mueller [jmueller@swiftk.com] -->
<!-- This code ported from C# by Jimmy Creel [jimmy_creel@hotmail.com] -->
<!-- Updated by Matthias Hess [matthias.hess@bluewin.ch] to fix XSLT "garbage" problem -->
<!-- Updated by Tor-Erik Hagen [tor-erik@web-amp.com] to make it possible not to specify namespace -->
<xsl:transform version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>
  <xsl:output method='text'/>

  <xsl:template match="/">
	<xsl:apply-templates select='/typeSafeCollections/typeSafeCollection[templateKind="SortedList"]'/>
  </xsl:template>
  
  <xsl:template match='/typeSafeCollections/typeSafeCollection[templateKind="SortedList"]'>

<xsl:if test="collectionNamespace">
	<xsl:if test="string-length(collectionNamespace) > 0">
Namespace <xsl:value-of select='collectionNamespace' />
	</xsl:if>
</xsl:if>

	&lt;Serializable()&gt; Public Class <xsl:value-of select='collectionName' />
		Implements IDictionary, ICloneable


#Region "Member Variables"

		Private Const DEFAULT_CAPACITY As Integer = 16

		Private m_keys() As <xsl:value-of select='keyType' />		   
		Private m_values() As <xsl:value-of select='itemType' />		  
		Private m_count As Integer
		Private m_version As Integer
		Private m_comparer As IComparer
		Private m_keyList As KeyList
		Private m_valueList As ValueList

#End Region
#Region "Constructors"
		Public Sub New()
		'		Initializes a new instance of the <xsl:value-of select='collectionName'/> class that is empty, 
		'		has the default initial capacity and is sorted according to the IComparable interface implemented 
		'		by each key added to the <xsl:value-of select='collectionName'/>.
		
			ReDim m_keys(DEFAULT_CAPACITY)
			ReDim m_values(DEFAULT_CAPACITY)
			m_comparer = Comparer.Default
		End Sub

		Public Sub New(ByVal capacity As Integer)
		'		Initializes a new instance of the <xsl:value-of select='collectionName'/> class that is empty, has the specified 
		'		initial capacity and is sorted according to the IComparable interface implemented by each key added 
		'		to the <xsl:value-of select='collectionName'/>

		'		Parameters:
		'		capacity = The initial number of elements that the <xsl:value-of select='collectionName'/> can contain.

		'		Execptions thrown:
		'		ArgumentOutOfRangeException if capacity is less than zero.

			If (capacity &lt; 0) Then
				Throw New ArgumentOutOfRangeException("capacity", capacity, "Initial capacity cannot be less than zero.")
			End If

			ReDim m_keys(DEFAULT_CAPACITY)
			ReDim m_values(DEFAULT_CAPACITY)
			m_comparer = Comparer.Default
		End Sub

		Public Sub New(ByRef comparer As IComparer)
		'		Initializes a new instance of the <xsl:value-of select='collectionName'/> class that is empty, 
		'		has the default initial capacity and is sorted according to the IComparable interface implemented 
		'		by each key added to the <xsl:value-of select='collectionName'/>.
		
		'		Parameters:
		'		comparer = The IComparer implementation to use when comparing keys or a null reference, to use 
		'		the IComparable implementation of each key.

			Me.new()
			If Not (comparer Is Nothing) Then
				Me.m_comparer = comparer
			End If
		End Sub

		Public Sub New(ByRef comparer As IComparer, ByVal capacity As Integer)
		'		Initializes a new instance of the <xsl:value-of select='collectionName'/> class that is empty, has the specified 
		'		initial capacity and is sorted according to the IComparable interface implemented by each key added 
		'		to the <xsl:value-of select='collectionName'/>

		'		Parameters:
		'		capacity = The initial number of elements that the <xsl:value-of select='collectionName'/> can contain.

		'		comparer = The IComparer implementation to use when comparing keys or a null reference, to use 
		'		the IComparable implementation of each key.


			Me.new(capacity)
			If Not (comparer Is Nothing) Then
				Me.m_comparer = comparer
			End If
		End Sub

		Public Sub New(ByRef d As IDictionary)
		'		Initializes a new instance of the <xsl:value-of select='collectionName'/> class that contains 
		'		elements copied from the specified dictionary, has the same initial capacity as the number of elements 
		'		copied and is sorted according to the IComparable interface implemented by each key.

		'		Parameters:
		'		d = The IDictionary to copy to a new SortedList.

			Me.new(d, Nothing)
		End Sub

		Public Sub New(ByRef d As IDictionary, ByRef comparer As IComparer)
		'		Initializes a new instance of the <xsl:value-of select='collectionName'/> class that contains elements copied 
		'		from the specified dictionary, has the same initial capacity as the number of elements copied and is sorted 
		'		according to the specified Comparer interface.

		'		Parameters:
		'		d = The IDictionary to copy to a new SortedList.
		'		comparer = The IComparer implementation to use when comparing keys or a null reference, to use 
		'		the IComparable implementation of each key.

		'		Exceptions thrown:
		'		ArgumentNullException = d is a null reference.
		
			Me.new(comparer, CInt(IIf(d Is Nothing, 0, d.Count)))
			If (d Is Nothing) Then
				Throw New ArgumentNullException("d", "The IDictionary cannot be null.")
			End If

			d.Keys.CopyTo(m_Keys, 0)
			d.Values.CopyTo(m_Values, 0)
			Array.Sort(Me.m_Keys, Me.m_Values, Me.m_comparer)
			Me.m_count = d.Count
		End Sub
#End Region

#Region "Properties"
		Default Public Overridable Property Item(ByVal key As Object) As Object Implements IDictionary.Item
		'		Gets and sets the value associated with a specific key in the <xsl:value-of select='collectionName'/>
		'		Inputs:
		'		The <xsl:value-of select='itemType' /> associated with key in the <xsl:value-of select='collectionName'/> if the key is found; 
		'		otherwise, a NullReferenceException is thrown.

		'		remarks:
		'		This property, unlike its equivalent in System.Collections.SortedList, does not return
		'		null when a key cannot be found. The reason for this is that the strongly-typed sorted list
		'		template this class is created from allows the value to be a value type (or struct), which cannot
		'		be set to null. Instead, a NullReferenceException is thrown when a key cannot be found.
		'		Callers must either catch this exception, or call the <xsl:value-of select='collectionName'/>.Contains method first to
		'		see if the key exists.

		'		Exceptions thrown:
		'		ArgumentNullException" = The key is a null reference.
		'		ArgumentException = Either the key or the value are not of a type supported by the <xsl:value-of select='collectionName'/>.
		'		NullReferenceException = key" is not found in the <xsl:value-of select='collectionName'/>

			Get
				If key Is Nothing Then
					Throw New ArgumentNullException("key", "The key cannot be null.")
				End If
				If Not (TypeOf key Is <xsl:value-of select='keyType' />) Then
					Throw New ArgumentException("The key must be of type:" &amp; GetType( <xsl:value-of select='keyType' />).FullName, "key")
				End If
				Return Me.Item(CType(key, <xsl:value-of select='keyType' />))
			End Get

			Set(ByVal value as Object)
				If (key Is Nothing) Then
					Throw New ArgumentNullException("key", "The key cannot be null.")
				End If
				If Not (TypeOf key Is <xsl:value-of select='keyType' />) Then
					Throw New ArgumentException("The key must be of type: " &amp; GetType(<xsl:value-of select='keyType' />).FullName, "key")
				End If

				If Not (TypeOf value Is <xsl:value-of select='itemType' />) Then
					Throw New ArgumentException("The value must be of type: " &amp; GetType(<xsl:value-of select='itemType' />).FullName, "key")
				End If
				Me.Item(CType(key, <xsl:value-of select='keyType' />)) = CType(value, <xsl:value-of select='itemType' />)


			End Set

		End Property


		Default Public Overridable Property Item(ByVal key As <xsl:value-of select='keyType' />) As <xsl:value-of select='itemType' />
		'		Gets and sets the value associated with a specific key in the <xsl:value-of select='collectionName'/>.
		'		Inputs:
		'		The <xsl:value-of select='itemType' /> associated with key in the <xsl:value-of select='collectionName'/> if the key is found; 
		'		otherwise, a NullReferenceException is thrown.

		'		remarks:
		'		This property, unlike its equivalent in System.Collections.SortedList, does not return
		'		null when a key cannot be found. The reason for this is that the strongly-typed sorted list
		'		template this class is created from allows the value to be a value type (or struct), which cannot
		'		be set to null. Instead, a NullReferenceException is thrown when a key cannot be found.
		'		Callers must either catch this exception, or call the <xsl:value-of select='collectionName'/>.Contains method first to
		'		see if the key exists.

		'		Exceptions thrown:
		'		ArgumentNullException" = The key is a null reference.
		'		NullReferenceException = key" is not found in the <xsl:value-of select='collectionName'/>

			Get
				Dim index As Integer = IndexOfKey(key)
				If (index &gt;= 0) Then
					Return m_values(index)
				End If

				Throw New NullReferenceException("The specified key could not be found.")
			End Get
			Set(ByVal Value As <xsl:value-of select='itemType' />)
				If (Object.ReferenceEquals(key, Nothing)) Then				 '// avoids compiler error for null check on value type
					Throw New ArgumentNullException("key", "The key cannot be null.")
				End If

				Dim index As Integer = Array.BinarySearch(m_keys, 0, m_count, key, m_comparer)
				If (index &gt;= 0) Then
					m_values(index) = Value
					m_version += 1
					Return
				End If
				Insert(NOT(index), key, Value)
			End Set
		End Property

		Public Overridable Property Capacity() As Integer
		'		Gets or sets the capacity of the <xsl:value-of select='collectionName'/>.

		'		Inputs:
		'		value = The number of elements that the <xsl:value-of select='collectionName'/> can contain.
		
		'		Remarks:
		'		If the <xsl:value-of select='collectionName'/> already has elements in it, then a new array is created for the
		'		keys and values with the new capacity and the old ones are copied into the new arrays.  NOTE: If you have more
		'		elements in the old <xsl:value-of select='collectionName'/> than capacity then you will loose those elements 
		'		above capacity.
			Get
				Return m_keys.Length
			End Get
			Set(ByVal Value As Integer)
				If (Value &lt; m_count) Then Value = m_count

				If (Value &lt;&gt; m_keys.Length) Then
					If (Value &gt; 0) Then
						Dim newKeys(Value) As <xsl:value-of select='keyType' />
						Dim newValues(Value) As <xsl:value-of select='itemType' />

						If (m_count &gt; 0) Then
							Array.Copy(m_keys, 0, newKeys, 0, m_count)
							Array.Copy(m_values, 0, newValues, 0, m_count)

							m_keys = newKeys
							m_values = newValues
						Else
							ReDim m_keys(DEFAULT_CAPACITY)
							ReDim m_values(DEFAULT_CAPACITY)
						End If
					End If
				End If
			End Set
		End Property

		Protected Property version() As Integer
			Get
				Return m_version
			End Get
			Set(ByVal Value As Integer)
				m_version = Value
			End Set
		End Property

		Public Overridable ReadOnly Property Count() As Integer Implements IDictionary.Count
		'		Gets the number of elements contained in the <xsl:value-of select='collectionName'/>.
			Get
				Return m_count
			End Get
		End Property

		Public Overridable ReadOnly Property IsFixedSize() As Boolean Implements IDictionary.IsFixedSize
		'		Gets a value indicating whether the <xsl:value-of select='collectionName'/> has a fixed size.
		'		The default is false.
			Get
				Return False
			End Get
		End Property
		Public Overridable ReadOnly Property IsReadOnly() As Boolean Implements IDictionary.IsReadOnly
		'		Gets a value indicating whether the <xsl:value-of select='collectionName'/> is read only.
		'		The default is false.
			Get
				Return False
			End Get
		End Property
		Public Overridable ReadOnly Property IsSynchronized() As Boolean Implements IDictionary.IsSynchronized
		'		Gets a value indicating whether the <xsl:value-of select='collectionName'/> is Synchronized.
		'		The default is false.
			Get
				Return False
			End Get
		End Property
		Public Overridable ReadOnly Property Keys() As ICollection Implements IDictionary.Keys
		'		Gets the keys in the <xsl:value-of select='collectionName'/>.

		'		Returns an ICollection containing the keys in the <xsl:value-of select='collectionName'/>

		'		Remarks:
		'		The ICollection is a read-only view of the keys in the <xsl:value-of select='collectionName'/>. Modifications made 
		'		to the underlying <xsl:value-of select='collectionName'/> are immediately reflected in the ICollection.
		'		The elements of the ICollection are sorted in the same order as the keys of the <xsl:value-of select='collectionName'/>
		'		Similar to <xsl:value-of select='collectionName'/>.GetKeyList, but returns an ICollection instead of an IList.

			Get
				Return GetKeyList()
			End Get
		End Property
		Public Overridable ReadOnly Property SyncRoot() As Object Implements IDictionary.SyncRoot
		'		Gets an object that can be used to synchronize access to the <xsl:value-of select='collectionName'/>.
			Get
				Return Me
			End Get
		End Property
		Public Overridable ReadOnly Property Values() As ICollection Implements IDictionary.Values
		'		Gets the values in the <xsl:value-of select='collectionName'/>.

		'		Remarks:
		'		The ICollection is a read-only view of the values in the <xsl:value-of select='collectionName'/>. Modifications made
		'		to the underlying <xsl:value-of select='collectionName'/> are immediately reflected in the ICollection.
		'		The elements of the ICollection are sorted in the same order as the values of the <xsl:value-of select='collectionName'/>.
		'		Similar to <xsl:value-of select='collectionName'/>.GetValueList, but returns an ICollection instead of an IList.

			Get
				Return GetValueList()
			End Get
		End Property

#End Region

#Region "Public Methods"
		Public Overridable Sub Add(ByVal key As Object, ByVal value As Object) Implements IDictionary.Add
		'		Adds an element with the specified key and value to the <xsl:value-of select='collectionName'/>.

		'		Inputs:
		'		key = The key of the element to add.
		'		value = The value of the element to add.

		'		Exceptions thrown:
		'		ArgumentNullException = The key is a null reference.
		'		ArgumentException =An element with the specified key already exists in the <xsl:value-of select='collectionName'/> or
		'		Either the key or the value are not of a type supported by the <xsl:value-of select='collectionName'/> or the <xsl:value-of select='collectionName'/>
		'		is set to use the IComparable, and key does not implement the IComparable interface.
		'		InvalidOperationException = The comparer throws an exception.
		'		NotSupportedException = The <xsl:value-of select='collectionName'/> is read-only or the <xsl:value-of select='collectionName'/> has a fixed size.

			If (Object.ReferenceEquals(key, Nothing)) Then
				Throw New ArgumentNullException("key", "The key cannot be null.")
			End If

			If (Not (TypeOf key Is <xsl:value-of select='keyType' />)) Then
				Throw New ArgumentException("The key must be of type: " &amp; GetType(<xsl:value-of select='keyType' />).FullName, "key")
			End If


			If (Not (TypeOf value Is <xsl:value-of select='itemType' />)) Then
				Throw New ArgumentException("The value must be of type: " &amp; GetType(<xsl:value-of select='itemType' />).FullName, "value")
			End If


			Me.Add(CType(key, <xsl:value-of select='keyType' />), CType(value, <xsl:value-of select='itemType' />))
		End Sub
		Public Overridable Sub Add(ByVal key As <xsl:value-of select='keyType' />, ByVal value As <xsl:value-of select='itemType' />)
		'		Adds an element with the specified key and value to the <xsl:value-of select='collectionName'/>.

		'		Inputs:
		'		key = The key of the element to add.
		'		value = The value of the element to add.

		'		Exception thrown:
 		'		ArgumentNullException = The key is a null reference.
		'		ArgumentException = An element with the specified key already exists in the <xsl:value-of select='collectionName'/> or
		'		the <xsl:value-of select='collectionName'/> is set to use the IComparable interface, and key does not implement the 
		'		IComparable interface.
		'		InvalidOperationException = The comparer throws an exception.
		'		NotSupportedException = The <xsl:value-of select='collectionName'/>is read-only or the <xsl:value-of select='collectionName'/>has a fixed size.

			If (Object.ReferenceEquals(key, Nothing)) Then			  '// avoids compiler error for null check on value type
				Throw New ArgumentNullException("key", "The key cannot be null.")
			End If

			Dim index As Integer = Array.BinarySearch(m_keys, 0, m_count, key, m_comparer)

			If (index &gt;= 0) Then
				Throw New ArgumentException("Item has already been added.  Key being added:" &amp; key &amp; ".")
			End If

			Insert(NOT(index), key, value)
		End Sub

		Public Overridable Sub Clear() Implements IDictionary.Clear
		'		Removes all elements from the <xsl:value-of select='collectionName'/>.

		'		Exception thrown:
		'		NotSupportedException = the <xsl:value-of select='collectionName'/> is read-only or the <xsl:value-of select='collectionName'/> has a fixed size.
			ReDim m_keys(DEFAULT_CAPACITY)
			ReDim m_values(DEFAULT_CAPACITY)
			m_count = 0
			m_version += 1
		End Sub

		Public Overridable Function Clone() As Object Implements ICloneable.Clone
		'		Creates a shallow copy of the <xsl:value-of select='collectionName'/>.
			Dim newList As <xsl:value-of select='collectionName' /> = New <xsl:value-of select='collectionName' />(m_comparer, m_count)
			Array.Copy(m_keys, 0, newList.m_keys, 0, m_count)
			Array.Copy(m_values, 0, newList.m_values, 0, m_count)
			newList.version = m_version
			Return newList
		End Function

		Public Overridable Function Contains(ByVal key As Object) As Boolean Implements IDictionary.Contains
		'		Determines whether the <xsl:value-of select='collectionName'/> contains a specific key.
		'		if the <xsl:value-of select='collectionName'/> contains an element with the specified key returns true, otherwise returns false.

		'		Inputs:
		'		key = The key to locate in the <xsl:value-of select='collectionName'/>.
		
		'		Exceptions thrown:
		'		ArgumentNullException = The key is a null reference.
		'		ArgumentException = key is not of a type supported by the <xsl:value-of select='collectionName'/>.
		'		InvalidOperationException = The comparer throws an exception.

			If (key Is Nothing) Then
				Throw New ArgumentNullException("key", "The key cannot be null.")
			End If

			If Not (TypeOf key Is <xsl:value-of select='itemType' />) Then
				Throw New ArgumentException("The key must be of type: " &amp; GetType(<xsl:value-of select='keyType' />).FullName, "key")
			End If

			Return (IndexOfKey(ctype(key,<xsl:value-of select='keyType' />)) &gt;= 0)
		End Function

		Public Overridable Function Contains(ByVal key As <xsl:value-of select='keyType' />) As Boolean
		'		Determines whether the <xsl:value-of select='collectionName'/> contains a specific key.
		
		'		Inputs:
		'		key = The key to locate in the <xsl:value-of select='collectionName'/>.

			Return (IndexOfKey(key) &gt;= 0)
		End Function

		Public Overridable Function ContainsKey(ByVal key As <xsl:value-of select='keyType' />) As Boolean
		'		Determines whether the <xsl:value-of select='collectionName'/> contains a specific key.

		'		Inputs:
		'		key = The key to locate in the <xsl:value-of select='collectionName'/>.
		
		'		Exceptions thrown:
		'		ArgumentNullException = The key is a null reference.
		'		InvalidOperationException = The comparer throws an exception.

			Return (IndexOfKey(key) &gt;= 0)
		End Function

		Public Overridable Function ContainsValue(ByVal value As <xsl:value-of select='itemType' />) As Boolean
		'		Determines whether the <xsl:value-of select='collectionName'/> contains a specific value.

		'		Inputs:
		'		value = The value to locate in the <xsl:value-of select='collectionName'/>.

			Return (IndexOfValue(value) &gt;= 0)
		End Function

		Public Overridable Function IEnumerable_GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
		'		Returns an IEnumerator that can iterate through the <xsl:value-of select='collectionName'/>.
			Return New SortedListEnumerator(Me, 0, m_count, SortedListEnumerator.DictEntry)
		End Function

		Public Overridable Function IDictionary_GetEnumerator() As IDictionaryEnumerator Implements IDictionary.GetEnumerator
		'		Returns an IDictionaryEnumerator that can iterate through the <xsl:value-of select='collectionName'/>.		
			Return New SortedListEnumerator(Me, 0, m_count, SortedListEnumerator.DictEntry)
		End Function

		Public Overridable Sub CopyTo(ByVal array As Array, ByVal arrayIndex As Integer) Implements IDictionary.CopyTo
		'		Copies the <xsl:value-of select='collectionName'/> elements to a one-dimensional System.Array
		'		instance at the specified index.

		'		Inputs:
		'		array = The one-dimensional System.Array that is the destination of the DictionaryEntry objects copied from <xsl:value-of select='collectionName'/>.
		'		The array must have zero-based indexing.
		'		arrayIndex = The zero-based index in array at which copying begins.

		'		Exceptions thrown:
		'		ArgumentNullException = array is a null reference.
		'		ArgumentOutOfRangeException = arrayIndex is less than zero.
		'		ArgumentException array is multidimensional or 	arrayIndex is equal to or greater than the length of array -or
		'		the number of elements in the source <xsl:value-of select='collectionName'/> is greater than the 
		'		available space from arrayIndex to the end of the destination array.
		'		InvalidCastException = The type of the source <xsl:value-of select='collectionName'/> cannot be cast automatically to the type
		'		of the destination array.
			If (array Is Nothing) Then
				Throw New ArgumentNullException("array", "The destination array cannot be null.")
			End If
			If (arrayIndex &lt; 0) Then
				Throw New ArgumentOutOfRangeException("arrayIndex", "Destination index cannot be less than zero.")
			End If
			If (array.Rank &lt;&gt; 1) Then
				Throw New ArgumentException("Multidimensional arrays are not supported.", "array")
			End If
			If (arrayIndex &gt;= array.Length) Then
				Throw New ArgumentException("Destination index cannot be greater than the size of the destination array.", "arrayIndex")
			End If
			If (m_count &gt; (array.Length - arrayIndex)) Then
				Throw New ArgumentException("Not enough available space in the destination array.")
			End If
			Dim i As Integer
			For i = 0 To m_count - 1
				Dim entry As DictionaryEntry = New DictionaryEntry(m_keys(i), m_values(i))
				array.SetValue(entry, arrayIndex + i)
			Next
		End Sub

		Public Overridable Function GetByIndex(ByVal index As Integer) As <xsl:value-of select='itemType' />
		'		Gets the value at the specified index of the <xsl:value-of select='collectionName'/>.

		'		Inputs:
		'		index = The zero-based index of the value to get.

		'		Exceptions thrown:
		'		ArgumentOutOfRangeException = index is outside the range of valid indices for the <xsl:value-of select='collectionName'/>.
			If (index &lt; 0 Or index &gt;= m_count) Then
				Throw New ArgumentOutOfRangeException("index", index, "The index is outside the range of valid indices.")
			End If
			Return m_values(index)
		End Function

		Public Overridable Function GetKey(ByVal index As Integer) As <xsl:value-of select='keyType' />
		'		Gets the key at the specified index of the <xsl:value-of select='collectionName'/>.

		'		Inputs:
		'		index = The zero-based index of the key to get.

		'		Exceptions thrown:
		'		ArgumentOutOfRangeException = index is outside the range of valid indices for the <xsl:value-of select='collectionName'/>.
			If (index &lt; 0 Or index &gt;= m_count) Then
				Throw New ArgumentOutOfRangeException("index", index, "The index is outside the range of valid indices.")
			End If
			Return m_keys(index)
		End Function

		Public Overridable Function GetKeyList() As IList
		'		Gets the keys in the <xsl:value-of select='collectionName'/>.

		'		Remarks:
		'		The returned IList is a read-only view of the keys in the <xsl:value-of select='collectionName'/>. Modifications made to 
		'		the underlying <xsl:value-of select='collectionName'/> are immediately reflected in the IList.
		'		The elements of the IList are sorted in the same order as the keys 	of the <xsl:value-of select='collectionName'/>.
		'		Similar to <xsl:value-of select='collectionName'/>.Keys, but returns an IList instead of an ICollection.
			If (m_keyList Is Nothing) Then m_keyList = New KeyList(Me)

			Return m_keyList
		End Function

		Public Overridable Function GetValueList() As IList
		'		Gets the values in the <xsl:value-of select='collectionName'/>.

		'		Rremarks:
		'		The returned IList is a read-only view of the values in the <xsl:value-of select='collectionName'/>. Modifications made to 
		'		the underlying <xsl:value-of select='collectionName'/> are immediately reflected in the IList.
		'		The elements of the IList are sorted in the same order as the values of the <xsl:value-of select='collectionName'/>.
		'		Similar to <xsl:value-of select='collectionName'/>.Values, but returns an IList instead of an ICollection.
			If (m_valueList Is Nothing) Then m_valueList = New ValueList(Me)
			Return m_valueList
		End Function


		Public Overridable Sub Remove(ByVal key As Object) Implements IDictionary.Remove
		'		Removes the element with the specified key from the <xsl:value-of select='collectionName'/>.

		'		Inputs:
		'		key = The key of the element to remove.

		'		Exceptions thrown;
		'		ArgumentException = The key is not of a type supported by the <xsl:value-of select='collectionName'/>.
		'		ArgumentNullException = key is a null reference.
		'		NotSupportedException = The <xsl:value-of select='collectionName'/> is read-only or the <xsl:value-of select='collectionName'/> has a fixed size.
			If (key Is Nothing) Then
				Throw New ArgumentNullException("key", "The key cannot be null.")
			End If

			If Not (TypeOf key Is <xsl:value-of select='keyType' />) Then
				Throw New ArgumentException("The key must be of type: " + GetType(<xsl:value-of select='keyType' />).FullName, "key")
			End If

			Remove(CType(key, <xsl:value-of select='itemType' />))
		End Sub

		Public Overridable Function IndexOfKey(ByVal key As <xsl:value-of select='keyType' />) As Integer
		'		Returns the zero-based index of the specified key in the <xsl:value-of select='collectionName'/>.

		'		Inputs:
		'		key = The key to locate in the <xsl:value-of select='collectionName'/>.

		'		Exceptions thrown:
		'		ArgumentNullException = key is a null reference.
		'		InvalidOperationException = The comparer throws an exception.

		'		Remarks:
		'		The elements of a <xsl:value-of select='collectionName'/> are sorted by the keys either according to a specific IComparer 
		'		implementation specified when the <xsl:value-of select='collectionName'/> is created or according to the IComparable implementation 
		'		provided by the keys themselves. The index sequence is based on the sort sequence. When an element is added, it is inserted into 
		'		<xsl:value-of select='collectionName'/> in the correct sort order, and the indexing adjusts accordingly. When an element removed, 
		'		the indexing also adjusts accordingly.  Therefore, the index of a specific key-and-value pair might change as elements are added or 
		'		removed from the <xsl:value-of select='collectionName'/>.  This method uses a binary search algorithm; therefore, the average 
		'		execution time is proportional to Log2(&lt;i&gt;n&lt;/i&gt;), where &lt;i&gt;n&lt;/i&gt; is <xsl:value-of select='collectionName'/>.Count.
			If (Object.ReferenceEquals(key, Nothing)) Then			  '// avoids compiler error for null check on value type
				Throw New ArgumentNullException("key", "The key cannot be null.")
			End If

			Dim index As Integer = Array.BinarySearch(m_keys, 0, m_count, key, m_comparer)

			Return CInt(IIf(index &gt;= 0, index, -1))
		End Function

		Public Overridable Function IndexOfValue(ByVal value As <xsl:value-of select='itemType' />) As Integer
		'		Returns the zero-based index of the first occurrence of the specified value in the <xsl:value-of select='collectionName'/>.

		'		Inputs:
		'		value = The value to locate in the <xsl:value-of select='collectionName'/>.

		'		Remarks:
		'		The index sequence is based on the sort sequence. When an element is added, 
		'		it is inserted into <xsl:value-of select='collectionName'/> in the correct sort order, and 
		'		the indexing adjusts accordingly. When an element removed, the indexing also adjusts 
		'		accordingly. Therefore, the index of a specific key-and-value pair might change as 
		'		elements are added or removed from the <xsl:value-of select='collectionName'/>.
		'		The values of the elements of the <xsl:value-of select='collectionName'/> are compared to the 
		'		specified value using the Equals method.
		'		This method uses a linear search; therefore, the average execution time is 
		'		proportional to <xsl:value-of select='collectionName'/>.Count.
			Return Array.IndexOf(m_values, value, 0, m_count)
		End Function

		Public Overridable Sub Remove(ByVal key As <xsl:value-of select='keyType' />)
		'		Removes the element with the specified key from the <xsl:value-of select='collectionName'/>.

		'		Input:
		'		key = The key of the element to remove.

		'		Exceptions thrown:
		'		ArgumentNullException = key is a null reference.
		'		NotSupportedException = The <xsl:value-of select='collectionName'/> is read-only or the <xsl:value-of select='collectionName'/> has a fixed size.
		' 
		'		Remarks:
		'		If the <xsl:value-of select='collectionName'/> does not contain an element with the specified key,
		'		the <xsl:value-of select='collectionName'/> remains unchanged. No exception is thrown.
			If (Object.ReferenceEquals(key, Nothing)) Then			  '// avoids compiler error for null check on value type
				Throw New ArgumentNullException("key", "The key cannot be null.")
			End If

			Dim index As Integer = IndexOfKey(key)
			If (index &gt;= 0) Then RemoveAt(index)
		End Sub

		Public Overridable Sub RemoveAt(ByVal index As Integer)
		'		Removes the element at the specified index of the <xsl:value-of select='collectionName'/>.

		'		Inputs:
		'		index = The zero-based index of the element to remove.
		
		'		Exceptions thrown:
		'		ArgumentOutOfRangeException = index is outside the range of valid indices for the <xsl:value-of select='collectionName'/>.
		'		NotSupportedException = The <xsl:value-of select='collectionName'/> is read-only or the <xsl:value-of select='collectionName'/> has a fixed size.
		' 
		'		Remarks:
		'		The index sequence is based on the sort sequence. When an element is added, 
		'		it is inserted into <xsl:value-of select='collectionName'/> in the correct sort order, and 
		'		the indexing adjusts accordingly. When an element removed, the indexing also adjusts 
		'		accordingly. Therefore, the index of a specific key-and-value pair might change as 
		'		elements are added or removed from the <xsl:value-of select='collectionName'/>.
		'		In collections of contiguous elements, such as lists, the elements that
		'		follow the removed element move up to occupy the vacated spot. If the collection is
		'		indexed, the indices of the elements that are moved are also updated.
		
			If (index &lt; 0 Or index &gt;= m_count) Then
				Throw New ArgumentOutOfRangeException("index", index, "The index is outside the range of valid indices.")
			End If

			m_count -= 1
			If (index &lt; m_count) Then
				Array.Copy(m_keys, index + 1, m_keys, index, m_count - index)
				Array.Copy(m_values, index + 1, m_values, index, m_count - index)
			End If

			' We can't set the deleted entries equal to null, because they might be value types.
			' Instead, we'll create empty single-element arrays of the right type and copy them 
			' over the entries we want to erase.
			Dim tempKey(1) As <xsl:value-of select='keyType' />
			Dim tempVal(1) As <xsl:value-of select='itemType' />
			Array.Copy(tempKey, 0, m_keys, m_count, 1)
			Array.Copy(tempVal, 0, m_values, m_count, 1)

			m_version += 1
		End Sub

		Public Overridable Sub SetByIndex(ByVal index As Integer, ByVal value As <xsl:value-of select='itemType' />)
		'		Replaces the value at a specific index in the <xsl:value-of select='collectionName'/>.

		'		Inputs:
		'		index = The zero-based index at which to save value.
		'		value = The <xsl:value-of select='itemType' /> to save into the <xsl:value-of select='collectionName'/>.
	
		'		Exceptions thrown:
		'		ArgumentOutOfRangeException = index is outside the range of valid indices for the <xsl:value-of select='collectionName'/>.
		' 
		'		Remarks:
		'		The index sequence is based on the sort sequence. When an element is added, 
		'		it is inserted into <xsl:value-of select='collectionName'/> in the correct sort order, and 
		'		the indexing adjusts accordingly. When an element is removed, the indexing also adjusts 
		'		accordingly. Therefore, the index of a specific key-and-value pair might change as 
		'		elements are added or removed from the <xsl:value-of select='collectionName'/>.

			If (index &lt; 0 Or index &gt;= m_count) Then
				Throw New ArgumentOutOfRangeException("index", index, "The index is outside the range of valid indices.")
			End If
			m_values(index) = value
			m_version += 1
		End Sub

		Public Function Synchronized(ByVal list As <xsl:value-of select='collectionName' />) As <xsl:value-of select='collectionName' />
		'		Returns a synchronized (thread-safe) wrapper for the <xsl:value-of select='collectionName'/>.

		'		Inputs:
		'		list = The <xsl:value-of select='collectionName'/> to synchronize.

		'		Exceptions thrown:
		'		ArgumentNullException = list is a null reference.

			If (list Is Nothing) Then
				Throw New ArgumentNullException("list", "The list cannot be null.")
			End If
			Return New SyncSortedList(list)

		End Function
		Public Overridable Sub TrimToSize()
		'		Sets the capacity to the actual number of elements in the <xsl:value-of select='collectionName'/>.

		'		Exceptions thrown:
		'		NotSupportedException = The <xsl:value-of select='collectionName'/> is read-only or the <xsl:value-of select='collectionName'/> has a fixed size.
			Me.Capacity = m_count
		End Sub
#End Region
#Region "Private Methods"
		Private Sub Insert(ByVal index As Integer, ByVal key As <xsl:value-of select='keyType' />, ByVal value As <xsl:value-of select='itemType' />)
			If (Count = m_keys.Length) Then
				EnsureCapacity(Count + 1)
			End If

			If (index &lt; Count) Then
				Array.Copy(m_keys, index, m_keys, index + 1, Count - index)
				Array.Copy(m_values, index, m_values, index + 1, Count - index)
			End If
			m_keys(index) = key
			m_values(index) = value
			m_count += 1
			m_version += 1
		End Sub

		Private Sub EnsureCapacity(ByVal min As Integer)
			Dim newCapacity As Integer
			If m_keys.Length = 0 then 
				newcapacity = DEFAULT_CAPACITY
			Else
				newcapacity=  m_keys.Length * 2
			End If
			If (newCapacity &lt; min) Then
				newCapacity = min
			End If
			Me.Capacity = newCapacity
		End Sub
#End Region
#Region "Nested Class: SyncSortedList"
		&lt;Serializable()&gt; Private Class SyncSortedList
			Inherits <xsl:value-of select='collectionName' />


			Private list As <xsl:value-of select='collectionName' />
			Private root As Object

			Private Sub New()
			End Sub

			Public Sub New(ByVal list As <xsl:value-of select='collectionName' />)
				Me.list = list
				Me.root = list.SyncRoot
			End Sub
			Public Overloads Overrides Property Capacity() As Integer
				Get
					SyncLock (root)
						Return list.Capacity
					End SyncLock
				End Get
				Set(ByVal Value As Integer)
					SyncLock (root)
						list.Capacity = Value
					End SyncLock
				End Set
			End Property
			Public Overloads Overrides ReadOnly Property Count() As Integer
				Get
					SyncLock (root)
						Return list.Count
					End SyncLock
				End Get
			End Property
			Public Overloads Overrides ReadOnly Property IsFixedSize() As Boolean
				Get
					Return list.IsFixedSize
				End Get
			End Property
			Public Overloads Overrides ReadOnly Property IsReadOnly() As Boolean
				Get
					Return list.IsReadOnly
				End Get
			End Property
			Public Overloads Overrides ReadOnly Property IsSynchronized() As Boolean
				Get
					Return True
				End Get
			End Property
			Public Overloads Overrides ReadOnly Property SyncRoot() As Object
				Get
					Return root
				End Get
			End Property
			Default Public Overloads Overrides Property Item(ByVal key As Object) As Object
				Get
					SyncLock (root)
						Return list.Item(key)
					End SyncLock
				End Get
				Set(ByVal Value As Object)
					SyncLock (root)
						list.Item(key) = Value
					End SyncLock
				End Set

			End Property
			Default Public Overloads Overrides Property Item(ByVal key As <xsl:value-of select='keyType' />) As <xsl:value-of select='itemType' />
				Get
					SyncLock (root)
						Return list.Item(key)
					End SyncLock
				End Get
				Set(ByVal Value As <xsl:value-of select='itemType' />)
					SyncLock (root)
						list.Item(key) = Value
					End SyncLock
				End Set
			End Property
			Public Overloads Overrides Sub Add(ByVal key As Object, ByVal value As Object)
				SyncLock (root)
					list.Add(key, value)
				End SyncLock
			End Sub

			Public Overloads Overrides Sub Add(ByVal key As <xsl:value-of select='keyType' />, ByVal value As <xsl:value-of select='itemType' />)
				SyncLock (root)
					list.Add(key, value)
				End SyncLock
			End Sub

			Public Overloads Overrides Sub Clear()
				SyncLock (root)
					list.Clear()
				End SyncLock
			End Sub

			Public Overloads Overrides Function Clone() As Object
				SyncLock (root)
					Return list.Clone()
				End SyncLock
			End Function
			Public Overloads Overrides Function Contains(ByVal key As Object) As Boolean
				SyncLock (root)
					Return list.Contains(key)
				End SyncLock

			End Function

			Public Overloads Overrides Function Contains(ByVal key As <xsl:value-of select='keyType' />) As Boolean
				SyncLock (root)
					Return list.Contains(key)
				End SyncLock

			End Function
			Public Overloads Overrides Function ContainsKey(ByVal key As <xsl:value-of select='keyType' />) As Boolean
				SyncLock (root)
					Return list.ContainsKey(key)
				End SyncLock
			End Function
			Public Overloads Overrides Function ContainsValue(ByVal value As <xsl:value-of select='itemType' />) As Boolean
				SyncLock (root)
					Return list.ContainsValue(value)
				End SyncLock
			End Function
			Public Overloads Overrides Sub CopyTo(ByVal array As Array, ByVal index As Integer)
				SyncLock (root)
					list.CopyTo(array, index)
				End SyncLock
			End Sub
			Public Overloads Overrides Function GetByIndex(ByVal index As Integer) As <xsl:value-of select='itemType' />
				SyncLock (root)
					Return list.GetByIndex(index)
				End SyncLock
			End Function

			Public Overrides Function IDictionary_GetEnumerator() As IDictionaryEnumerator
				SyncLock (root)
					Return list.IDictionary_GetEnumerator
				End SyncLock
			End Function

			Public Overloads Overrides Function GetKey(ByVal index As Integer) As <xsl:value-of select='keyType' />
				SyncLock (root)
					Return list.GetKey(index)
				End SyncLock
			End Function
			Public Overloads Overrides Function GetKeyList() As IList
				SyncLock (root)
					Return list.GetKeyList()
				End SyncLock
			End Function
			Public Overloads Overrides Function GetValueList() As IList
				SyncLock (root)
					Return list.GetValueList()
				End SyncLock
			End Function
			Public Overloads Overrides Function IndexOfKey(ByVal key As <xsl:value-of select='keyType' />) As Integer
				SyncLock (root)
					Return list.IndexOfKey(key)
				End SyncLock
			End Function
			Public Overloads Overrides Function IndexOfValue(ByVal value As <xsl:value-of select='itemType' />) As Integer
				SyncLock (root)
					Return list.IndexOfValue(value)
				End SyncLock
			End Function

			Public Overloads Overrides Sub Remove(ByVal key As Object)
				SyncLock (root)
					list.Remove(key)
				End SyncLock
			End Sub

			Public Overloads Overrides Sub Remove(ByVal key As <xsl:value-of select='keyType' />)
				SyncLock (root)
					list.Remove(key)
				End SyncLock
			End Sub

			Public Overloads Overrides Sub RemoveAt(ByVal index As Integer)
				SyncLock (root)
					list.RemoveAt(index)
				End SyncLock
			End Sub
			Public Overloads Overrides Sub SetByIndex(ByVal index As Integer, ByVal value As <xsl:value-of select='itemType' />)
				SyncLock (root)
					list.SetByIndex(index, value)
				End SyncLock
			End Sub
			Public Overloads Overrides Sub TrimToSize()
				SyncLock (root)
					list.TrimToSize()
				End SyncLock
			End Sub

			Public Overrides Function IEnumerable_GetEnumerator() As System.Collections.IEnumerator
				SyncLock (root)
					Return list.IEnumerable_GetEnumerator()
				End SyncLock
			End Function
		End Class
#End Region

#Region "Nested Class: SortedListEnumerator"
		&lt;Serializable()&gt; Private Class SortedListEnumerator
			Implements IDictionaryEnumerator, ICloneable

			Private m_list As <xsl:value-of select='collectionName' />
			Private m_key As <xsl:value-of select='keyType' />
			Private m_value As <xsl:value-of select='itemType' />
			Private m_index As Integer
			Private m_startIndex As Integer
			Private m_endIndex As Integer
			Private m_version As Integer
			Private m_currentValid As Boolean
			Private m_returnType As Integer

			Friend Const Keys As Integer = 1
			Friend Const Values As Integer = 2
			Friend Const DictEntry As Integer = 3

			Friend Sub New(ByVal list As <xsl:value-of select='collectionName' />, ByVal index As Integer, ByVal count As Integer, ByVal returnType As Integer)
				Me.m_list = list
				Me.m_index = index
				Me.m_startIndex = index
				Me.m_endIndex = index + count
				Me.m_version = list.version
				Me.m_returnType = returnType
				Me.m_currentValid = False
			End Sub


			Public Function Clone() As Object Implements ICloneable.Clone
				Return Me.MemberwiseClone()
			End Function

			Public ReadOnly Property IDictionaryEnumerator_Key() As Object Implements IDictionaryEnumerator.Key
				Get
					CheckState()
					Return m_key
				End Get
			End Property

			Public Overridable ReadOnly Property Key() As <xsl:value-of select='keyType' />
				Get
					CheckState()
					Return m_key
				End Get
			End Property

			Public Overridable ReadOnly Property Entry() As DictionaryEntry Implements IDictionaryEnumerator.Entry
				Get
					CheckState()
					Return New DictionaryEntry(m_key, m_value)
				End Get
			End Property

			Public Overridable ReadOnly Property Current() As Object Implements IDictionaryEnumerator.Current
				Get
					CheckState()

					Select Case (m_returnType)
						Case Keys
							Return Keys
						Case Values
							Return Values
						Case DictEntry
							Return New DictionaryEntry(m_key, m_value)
						Case Else
							Return New DictionaryEntry(m_key, m_value)
					End Select
				End Get
			End Property

			Public ReadOnly Property IDictionaryEnumerator_Value() As Object Implements IDictionaryEnumerator.Value
				Get
					CheckState()
					Return m_value
				End Get
			End Property

			Public Overridable ReadOnly Property Value() As <xsl:value-of select='itemType' />
				Get
					CheckState()
					Return m_value
				End Get
			End Property

			Public Overridable Function MoveNext() As Boolean Implements IDictionaryEnumerator.MoveNext
				If (m_version &lt;&gt; m_list.version) Then
					Throw New InvalidOperationException("The collection was modified - enumeration cannot continue.")
				End If

				If (m_index &lt; m_endIndex) Then
					m_key = m_list.m_keys(m_index)
					m_value = m_list.m_values(m_index)
					m_index += 1
					m_currentValid = True
					Return True
				End If
				'// We can't set the entries equal to null, because they might be value types.
				'// Instead, we'll create empty single-element arrays of the right type and copy them 
				'// over the entries we want to erase.
				Dim tempKey(1) As <xsl:value-of select='keyType' />
				Dim tempVal(1) As <xsl:value-of select='itemType' />
				m_key = tempKey(0)
				m_value = tempVal(0)
				m_currentValid = False
				Return False
			End Function
			Public Overridable Sub Reset() Implements IDictionaryEnumerator.Reset
				If (m_version &lt;&gt; m_list.version) Then
					Throw New InvalidOperationException("The collection was modified - enumeration cannot continue.")
				End If
				'// We can't set the entries equal to null, because they might be value types.
				'// Instead, we'll create empty single-element arrays of the right type and copy them 
				'// over the entries we want to erase.
				Dim tempKey(1) As <xsl:value-of select='keyType' />
				Dim tempVal(1) As <xsl:value-of select='itemType' />
				m_key = tempKey(0)
				m_value = tempVal(0)
				m_currentValid = False
				m_index = m_startIndex
			End Sub

			Private Sub CheckState()
				If (m_version &lt;&gt; m_list.version) Then
					Throw New InvalidOperationException("The collection was modified - enumeration cannot continue.")
				End If
				If Not (m_currentValid) Then
					Throw New InvalidOperationException("Enumeration either has not started or has already finished.")
				End If
			End Sub
		End Class
#End Region

#Region "Nested Class: KeyList"
		&lt;Serializable()&gt; Private Class KeyList
			Implements IList
			Private list As <xsl:value-of select='collectionName' />

			Protected Friend Sub New(ByVal list As <xsl:value-of select='collectionName' />)
				Me.list = list
			End Sub
			Public Overridable ReadOnly Property Count() As Integer Implements IList.Count
				Get
					Return list.Count
				End Get
			End Property

			Public Overridable ReadOnly Property IsReadOnly() As Boolean Implements IList.IsReadOnly
				Get
					Return True
				End Get
			End Property

			Public Overridable ReadOnly Property IsFixedSize() As Boolean Implements IList.IsFixedSize
				Get
					Return True
				End Get
			End Property

			Public Overridable ReadOnly Property IsSynchronized() As Boolean Implements IList.IsSynchronized
				Get
					Return list.IsSynchronized
				End Get
			End Property

			Public Overridable ReadOnly Property SyncRoot() As Object Implements IList.SyncRoot
				Get
					Return list.SyncRoot
				End Get
			End Property


			Public Overridable Function Add(ByVal key As Object) As Integer Implements IList.Add
				Throw New NotSupportedException("Cannot add to a read-only list.")
			End Function

			Public Overridable Sub Clear() Implements IList.Clear
				Throw New NotSupportedException("Cannot clear a read-only list.")
			End Sub

			Public Function Contains(ByVal key As Object) As Boolean Implements IList.Contains
				Return list.Contains(key)
			End Function

			Public Overridable Function Contains(ByVal key As <xsl:value-of select='keyType' />) As Boolean
				Return list.Contains(key)
			End Function

			Public Overridable Sub CopyTo(ByVal array As Array, ByVal index As Integer) Implements IList.CopyTo
				If (array Is Nothing And array.Rank &lt;&gt; 1) Then
					Throw New ArgumentException("Multidimensional arrays are not supported.", "array")
				End If

				array.Copy(list.m_keys, 0, array, index, list.Count)
			End Sub

			Public Overridable Sub Insert(ByVal index As Integer, ByVal value As Object) Implements IList.Insert
				Throw New NotSupportedException("Cannot insert into a read-only list.")
			End Sub

			Public Overridable Property IList_Item(ByVal index As Integer) As Object Implements IList.Item
				Get
					Return list.GetKey(index)
				End Get
				Set(ByVal Value As Object)
					Throw New NotSupportedException("Cannot modify a read-only list.")
				End Set
			End Property

			Public Overridable Property Item(ByVal index As Integer) As <xsl:value-of select='keyType' />
				Get
					Return list.GetKey(index)
				End Get
				Set(ByVal Value As <xsl:value-of select='keyType' />)
					Throw New NotSupportedException("Cannot modify a read-only list.")
				End Set
			End Property

			Public Overridable Function GetEnumerator() As IEnumerator Implements IList.GetEnumerator
				Return New SortedListEnumerator(list, 0, list.Count, SortedListEnumerator.Keys)
			End Function

			Public Function IndexOf(ByVal key As Object) As Integer Implements IList.IndexOf
				If Not (TypeOf key Is <xsl:value-of select='keyType' />) Then
					Throw New ArgumentException("The key must be of type: " &amp; GetType(<xsl:value-of select='keyType' />).FullName, "key")
				End If

				Return list.IndexOfKey(CType(key, <xsl:value-of select='keyType' />))
			End Function

			Public Overridable Function IndexOf(ByVal key As <xsl:value-of select='keyType' />) As Integer
				Return list.IndexOfKey(key)
			End Function


			Public Overridable Sub Remove(ByVal key As Object) Implements IList.Remove
				Throw New NotSupportedException("Cannot modify a read-only list.")
			End Sub
			Public Overridable Sub RemoveAt(ByVal index As Integer) Implements IList.RemoveAt
				Throw New NotSupportedException("Cannot modify a read-only list.")
			End Sub
		End Class
#End Region

#Region "Nested Class: ValueList"
		&lt;Serializable()&gt; Private Class ValueList
			Implements IList

			Private list As <xsl:value-of select='collectionName' />

			Private Sub New()
			End Sub

			Protected Friend Sub New(ByVal list As <xsl:value-of select='collectionName' />)
				Me.list = list
			End Sub

			Public Overridable ReadOnly Property Count() As Integer Implements IList.Count
				Get
					Return list.Count
				End Get
			End Property

			Public Overridable ReadOnly Property IsReadOnly() As Boolean Implements IList.IsReadOnly
				Get
					Return True
				End Get
			End Property

			Public Overridable ReadOnly Property IsFixedSize() As Boolean Implements IList.IsFixedSize
				Get
					Return True
				End Get
			End Property

			Public Overridable ReadOnly Property IsSynchronized() As Boolean Implements IList.IsSynchronized
				Get
					Return list.IsSynchronized
				End Get
			End Property

			Public Overridable ReadOnly Property SyncRoot() As Object Implements IList.SyncRoot
				Get
					Return list.SyncRoot
				End Get
			End Property

			Public Overridable Function Add(ByVal key As Object) As Integer Implements IList.Add
				Throw New NotSupportedException("Cannot add to a read-only list.")
			End Function

			Public Overridable Sub Clear() Implements IList.Clear
				Throw New NotSupportedException("Cannot clear a read-only list.")
			End Sub

			Public Function Contains(ByVal value As Object) As Boolean Implements IList.Contains
				If Not (TypeOf value Is <xsl:value-of select='itemType' />) Then
					Throw New ArgumentException("The value must be of type: " + GetType(<xsl:value-of select='itemType' />).FullName, "value")
				End If
				Return list.ContainsValue(CType(value, <xsl:value-of select='itemType' />))
			End Function

			Public Overridable Function Contains(ByVal value As <xsl:value-of select='itemType' />) As Boolean
				Return list.Contains(value)
			End Function

			Public Overridable Sub CopyTo(ByVal array As Array, ByVal index As Integer) Implements IList.CopyTo
				If (array Is Nothing And array.Rank &lt;&gt; 1) Then
					Throw New ArgumentException("Multidimensional arrays are not supported.", "array")
				End If
				array.Copy(list.m_values, 0, array, index, list.Count)
			End Sub

			Public Overridable Sub Insert(ByVal index As Integer, ByVal value As Object) Implements IList.Insert
				Throw New NotSupportedException("Cannot insert into a read-only list.")
			End Sub

			Public Property IList_Item(ByVal index As Integer) As Object Implements IList.Item
				Get
					Return list.GetByIndex(index)
				End Get
				Set(ByVal Value As Object)
					If Not (TypeOf Value Is <xsl:value-of select='itemType' />) Then
						Throw New ArgumentException("The value must be of type: " + GetType(<xsl:value-of select='itemType' />).FullName, "value")
					End If
					list.SetByIndex(index, CType(Value,<xsl:value-of select='itemType' />))
				End Set
			End Property

			Public Overridable Property Item(ByVal index As Integer) As <xsl:value-of select='itemType' />
				Get
					Return list.GetByIndex(index)
				End Get
				Set(ByVal Value As <xsl:value-of select='itemType' />)
					list.SetByIndex(index, Value)
				End Set
			End Property

			Public Overridable Function GetEnumerator() As IEnumerator Implements IList.GetEnumerator
				Return New SortedListEnumerator(list, 0, list.Count, SortedListEnumerator.Values)
			End Function

			Public Function IndexOf(ByVal value As Object) As Integer Implements IList.IndexOf
				If Not (TypeOf value Is <xsl:value-of select='itemType' />) Then
					Throw New ArgumentException("The value must be of type: " + GetType(<xsl:value-of select='itemType' />).FullName, "value")
				End If
				Return list.IndexOfValue(CType(value, <xsl:value-of select='itemType' />))
			End Function

			Public Overridable Function IndexOf(ByVal value As <xsl:value-of select='itemType' />) As Integer
				Return list.IndexOfValue(value)
			End Function

			Public Overridable Sub Remove(ByVal key As Object) Implements IList.Remove
				Throw New NotSupportedException("Cannot modify a read-only list.")
			End Sub

			Public Overridable Sub RemoveAt(ByVal index As Integer) Implements IList.RemoveAt
				Throw New NotSupportedException("Cannot modify a read-only list.")
			End Sub
		End Class
#End Region
	End Class
<xsl:if test="collectionNamespace">
	<xsl:if test="string-length(collectionNamespace) > 0">
End Namespace
	</xsl:if>
</xsl:if>

  </xsl:template>

</xsl:transform>

